@@ -45,6 +45,7 @@
 @class NSView;
 @class NSWindow;
 @class NSMutableArray;
+@class NSScreen;
 
 /* ******************* */
 /* NSMenuView Protocol */
@@ -65,12 +66,12 @@
  *  version and the transient version are on the screen.
  *  A value of -1 means that no item will be highlighted.
  */
-- (void) setHighlightedItemIndex: (int)index;
+- (void) setHighlightedItemIndex: (NSInteger)index;
 
 /** Returns the currently highlighted item.  Returns -1
  *  if no item is highlighted.
  */
-- (int) highlightedItemIndex;
+- (NSInteger) highlightedItemIndex;
 
 /** This should ensure that if there is an attached
  *  submenu this submenu will be detached.
@@ -122,7 +123,7 @@
  */
 - (NSPoint) locationForSubmenu: (NSMenu *)aSubmenu;
 
-- (void) performActionWithHighlightingForItemAtIndex: (int)index; //????
+- (void) performActionWithHighlightingForItemAtIndex: (NSInteger)index; //????
 
 /** <p>This is method is responsible for handling all events while
  *  the user is interacting with this menu.  It should pass on this
@@ -194,6 +195,63 @@
 
 @end
 
+/**
+ * The NSMenuDelegate protocol defines optional methods implemented
+ * by delegates of NSMenu objects.
+ */
+@protocol NSMenuDelegate <NSObject>
+
+/**
+ * Allows the delegate to return the target and action for a key-down event.
+ */
+- (BOOL) menuHasKeyEquivalent: (NSMenu *)menu
+                     forEvent: (NSEvent *)event
+                       target: (id *)target
+                       action: (SEL *)action;
+
+/**
+ * Invoked on the delegate to allow changes before the menu opens.
+ */
+- (void) menuWillOpen: (NSMenu *)menu;
+
+/**
+ * Invoked when the menu is about to be displayed.
+ */
+- (NSInteger) numberOfItemsInMenu: (NSMenu *)menu;
+
+/**
+ * Invoked to indicate that the menu is about to be updated.
+ */
+- (void) menuNeedsUpdate: (NSMenu *)menu;
+
+/**
+ * Invoked to inform the delegate that the menu did close.
+ */
+- (void) menuDidClose: (NSMenu *)menu;
+
+/**
+ * Invoked too notify the delegate that the item will be highlighted.
+ */
+- (void) menu: (NSMenu *)menu
+         willHighlightItem: (NSMenuItem *)item;
+
+/**
+ * Invoked to allow the delegate to update an item before
+ * it is displayed.
+ */ 
+- (BOOL) menu: (NSMenu *)menu
+   updateItem: (NSMenuItem *)item
+      atIndex: (NSInteger)index
+ shouldCancel: (BOOL)shouldCancel;
+
+/**
+ * Specify a display location for the menu
+ */
+- (NSRect)confinementRectForMenu: (NSMenu *)menu
+                        onScreen: (NSScreen *)screen;
+
+@end
+
 /** <p>Menus provide the user with a list of actions and/or submenus.
  *  Submenus themselves are full fledged menus and so a heirarchical
  *  structure of appears.</p>
@@ -336,18 +394,18 @@
   NSMenu *_superMenu;
   NSMenu *_attachedMenu;
   NSMutableArray *_notifications;
-	id _delegate;
+  id _delegate;
 
   // GNUstepExtra category
   NSPopUpButtonCell *_popUpButtonCell;
-	struct GSMenuFlags {
+  struct GSMenuFlags {
     unsigned int changedMessagesEnabled: 1;
     unsigned int autoenable: 1;
     unsigned int needsSizing: 1;
     unsigned int is_tornoff: 1;
     unsigned int transient: 1;
     unsigned int horizontal: 1;
-	unsigned int mainMenuChanged: 1;
+    unsigned int mainMenuChanged: 1;
 		unsigned int unused: 25;
   } _menu;
 
@@ -356,6 +414,7 @@
   NSWindow *_bWindow;
   NSMenu *_oldAttachedMenu;
   int     _oldHiglightedIndex;
+  NSString *_name;
 }
 
 /** Returns the memory allocation zone used to create instances of this class.
@@ -433,29 +492,29 @@
 
 /** Returns the index of item anObject.
  */
-- (int) indexOfItem: (id <NSMenuItem>)anObject;
+- (NSInteger) indexOfItem: (id <NSMenuItem>)anObject;
 
 /** Returns the index of an item with the tag aTag.
  */
-- (int) indexOfItemWithTag: (int)aTag;
+- (NSInteger) indexOfItemWithTag: (NSInteger)aTag;
 
 /** Returns the index of an item with the target anObject
  * and the actionSelector.
  */
-- (int) indexOfItemWithTarget: (id)anObject
+- (NSInteger) indexOfItemWithTarget: (id)anObject
                    andAction: (SEL)actionSelector;
 
 /** Returns the index of an item with the represented object anObject.
  */
-- (int) indexOfItemWithRepresentedObject: (id)anObject;
+- (NSInteger) indexOfItemWithRepresentedObject: (id)anObject;
 
 /** Returns the index of an item with the submenu anObject.
  */
-- (int) indexOfItemWithSubmenu: (NSMenu *)anObject;
+- (NSInteger) indexOfItemWithSubmenu: (NSMenu *)anObject;
 
 /** Returns the index of an item with the title aTitle.
  */
-- (int) indexOfItemWithTitle: (NSString *)aTitle;
+- (NSInteger) indexOfItemWithTitle: (NSString *)aTitle;
 
 /** <init/>
  */
@@ -464,7 +523,7 @@
 /** Insert newItem at position index.
  */
 - (void) insertItem: (id <NSMenuItem>)newItem
-            atIndex: (int)index;
+            atIndex: (NSInteger)index;
 
 /** Inserts a new menu item at position index.
  *  <p>See Also: 
@@ -476,7 +535,7 @@
 - (id <NSMenuItem>) insertItemWithTitle: (NSString *)aString
                                  action: (SEL)aSelector
                           keyEquivalent: (NSString *)charCode
-                                atIndex: (unsigned int)index;
+                                atIndex: (NSInteger)index;
 
 /** Returns if this menu is attached to its supermenu,
  *  return nil if it does not have a parent menu.
@@ -499,7 +558,7 @@
 
 /** Returns an item located at index.
  */
-- (id <NSMenuItem>) itemAtIndex: (int)index;
+- (NSMenuItem *) itemAtIndex: (NSInteger)index;
 
 /** Informs the menu that the specified item has changed.
  */
@@ -513,7 +572,7 @@
  *   <item>[(NSMenuItem)-tag]</item>
  *  </list>
  */
-- (id <NSMenuItem>) itemWithTag: (int)aTag;
+- (id <NSMenuItem>) itemWithTag: (NSInteger)aTag;
 
 /** Returns an item with aString as its title.
  */
@@ -542,7 +601,7 @@
 
 /** Returns the numbers of items on the menu
  */
-- (int) numberOfItems;
+- (NSInteger) numberOfItems;
 
 /** Simulates a mouse click on item located at index.
  *  <p>See Also:
@@ -552,7 +611,7 @@
  *   <item>-indexOfItemWithTitle:</item>
  *  </list>
  */
-- (void) performActionForItemAtIndex: (int)index;
+- (void) performActionForItemAtIndex: (NSInteger)index;
 
 /** Looks for a menu item that responds to theEvent on the receiver.  If 
  *  the receiver is a submenu, the method is performed on it.
@@ -565,7 +624,7 @@
 
 /** Removes item at position index.
  */
-- (void) removeItemAtIndex: (int)index;
+- (void) removeItemAtIndex: (NSInteger)index;
 
 /** Sets if a menu does autoenable.
  */
@@ -664,10 +723,10 @@
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
 @interface NSObject (NSMenuDelegate)
 - (void) menuNeedsUpdate: (NSMenu *)menu;
-- (int) numberOfItemsInMenu: (NSMenu *)menu;
+- (NSInteger) numberOfItemsInMenu: (NSMenu *)menu;
 - (BOOL) menu: (NSMenu *)menu
    updateItem: (NSMenuItem *)item
-      atIndex: (int)index
+      atIndex: (NSInteger)index
  shouldCancel: (BOOL)shouldCancel;
 - (BOOL) menuHasKeyEquivalent: (NSMenu *)menu
                      forEvent: (NSEvent *)event
