@@ -333,6 +333,19 @@ PACKAGE_SCOPE
  */
 - (NSPoint) convertScreenToBase: (NSPoint)aPoint;
 
+/**
+ * Converts aRect from the coordinate system of the screen
+ * to the coordinate system of the window.
+ */
+
+- (NSRect) convertRectFromScreen: (NSRect)aRect;
+
+/**
+ * Converts aRect from the window coordinate system to a rect in
+ * the screen coordinate system.
+ */
+- (NSRect) convertRectToScreen: (NSRect)aRect;
+
 /**
  * Returns the frame of the receiver ... the rectangular area that the window
  * (including any border, title, and other decorations) occupies on screen.
@@ -811,8 +824,15 @@ PACKAGE_SCOPE
  * Implemented by the delegate
  */
 
-#ifdef GNUSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
+@protocol NSWindowDelegate <NSObject>
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST) && GS_PROTOCOLS_HAVE_OPTIONAL
+@optional
+#else
+@end
 @interface NSObject (NSWindowDelegate)
+#endif
+
 - (BOOL) windowShouldClose: (id)sender;
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (void) windowWillBeginSheet: (NSNotification*)aNotification;
@@ -828,9 +848,6 @@ PACKAGE_SCOPE
 willPositionSheet: (NSWindow *)sheet
         usingRect: (NSRect)rect;
 #endif
-#if OS_API_VERSION(MAC_OS_X_VERSION_10_1, GS_API_LATEST)
-- (NSWindow *) attachedSheet;
-#endif
 - (NSSize) windowWillResize: (NSWindow*)sender
 		     toSize: (NSSize)frameSize;
 - (id) windowWillReturnFieldEditor: (NSWindow*)sender
@@ -855,6 +872,12 @@ willPositionSheet: (NSWindow *)sheet
 @end
 #endif
 
+@interface NSObject (NSWindowDelegateAdditions) <NSWindowDelegate>
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_1, GS_API_LATEST)
+- (NSWindow *) attachedSheet;
+#endif
+@end
+
 /* Notifications */
 APPKIT_EXPORT NSString *NSWindowDidBecomeKeyNotification;
 APPKIT_EXPORT NSString *NSWindowDidBecomeMainNotification;
